# /u/sy/beebe/src/psposter/psposter-0.11/a2ps-encoding-to-enc.awk, Wed Jan 12 10:00:46 2000
# Edit by Nelson H. F. Beebe <beebe@math.utah.edu>
# ======================================================================
# Convert a PostScript encoding specification from an a2ps encoding file
# to a .enc file in TeX format.
#
# Usage:
#	awk -f a2ps-encoding-to-enc.awk infile(s)
#
# The output files are automically named by the Name: entry in the input
# file(s).
# 
# [12-Jan-2000]
# ======================================================================

BEGIN				{ initialize() }

FILENAME != LAST_FILENAME	{ new_file() }

/Name:/				{ Name = $2; Outfile = Name ".enc" }


/^Vector:/			{ new_vector(); next }

InVector \
	{
	    for (k = 1; k <= NF; ++k)
		output("/" $k)
	}

END	{ new_file() }

#=======================================================================

function initialize()
{
    new_file()  
    Current_Date_and_Time = shell_command("date")
}


function new_file()
{
    if (NChars > 0)				    
    {
	print "] def" > Outfile
	print "%%EndResource" > Outfile
	close(Outfile)
	if (NChars != 256)
	    warning("Bad encoding vector: found " NChars " characters instead of expected 256")
    }
    LAST_FILENAME = FILENAME
    NChars = 0
    Outfile = "/dev/null"
    InVector = 0 
}


function new_vector()
{
    InVector = 1
    if ("USER" in ENVIRON)
	username = ENVIRON["USER"]
    else if ("LOGNAME" in ENVIRON)
	username = ENVIRON["LOGNAME"]
    else
	username = "unknown"
    personal_name = shell_command("awk -F: '/^" username ":/ {print $5}' /etc/passwd")
    if (personal_name == "")
	personal_name = shell_command("ypcat passwd | awk -F: '/^" username ":/ {print $5}'")
    hostname = shell_command("hostname")

    print "%!" > Outfile
    print "%%% This is psposter PostScript font encoding file", Outfile > Outfile
    print "" > Outfile
    print "%%%=====================================================================" > Outfile
    print "%%% Warning: Do NOT edit this file.  It was created automatically" > Outfile
    print "%%% by awk -f a2ps-encoding-to-enc.awk", FILENAME > Outfile
    print "%%% in", ENVIRON["PWD"] > Outfile
    print "%%% on", Current_Date_and_Time > Outfile
    print "%%% for", personal_name, "<" username "@" hostname ">" > Outfile
    print "%%%=====================================================================" > Outfile
    print "" > Outfile
    print "%%BeginResource: Encoding", Name > Outfile
    print "/" Name "-Encoding [" > Outfile
}


function output(word)
{
    if (NChars == 0) 
	Line_Length = 0
    if ((NChars % 16) == 0)
    {
	if (Line_Length > 0)
	    print "" > Outfile
	printf("    %% \\%03o  %3d  0x%02x\n", NChars, NChars, NChars) > Outfile
	Line_Length = 0
    }
    if ((NChars % 4) == 0)
    {
	printf("\t") > Outfile
	Line_Length = 8
    }
    printf("%s%s", word, ((NChars % 4) < 3) ? ((length(word) < 8) ? "\t\t" : "\t") : "") > Outfile
    Line_Length += 16
    if ((NChars % 4) == 3)
    {	
	print "" > Outfile
	Line_Length = 0
    }
    NChars++
}


function shell_command(command, s)
{
    ## Run a shell command and return its first output line, after
    ## closing the file.
    command | getline s
    close(command)
    return (s)
}


function warning(message)
{
    print FILENAME ":" FNR ":" message >"/dev/stderr"
}
